// --- [ logger_main.c ] -------------------------------------------------------
// Langzeitlogger mit
// - PCF8583 als Timer: fr die Messintervalle zustndig
// - TWI_eeprom als Datenspeicher
// - Mega88 als Controller (hauptschlich im sleep-Modus)

// neu:
// bei Start testen, ob alle angemeldeten Sensoren antworten

#define EXTERN

#include <avr/io.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#include <util/delay.h>
#include "PCF8583.h"
#include "rs232_input.h"
#include "TWI_Master_c.h"
#include "global.h"
#include "messung.h"
#include "owi_lib.h"

#define PIN_INT0        PD2					// wird vom INT des PCF8583 bedient
#define PIN_INT1        PD3					// fr das Beenden des Loggens

#define LED_LOG			PD7					// Logger aktiv
#define LED_KONFIG		PD6					// Konfiguration aktiv

// --- lokale Funktionen ----------------------------------------------------

void 	get_time(void);
void	get_value(uint8_t * data, uint8_t min, uint8_t max);
void 	get_single_char(uint8_t * vorgabe);
void 	start_logging(void);
void 	PCF8583_konfig(void);

// ----------------------------------------------------------------------------
int main(void)
{
uint8_t buffer[3], i;

// als "Hallo World" wird hier kurz an allen LED's gewackelt

// LED-Pins als Ausgang schalten, die LED's leuchten nun
DDRD  |= (1<<LED_KONFIG | 1<<LED_LOG );
delay_10ms(100);
// LED's nach 1 Sekunde abschalten
DDRD  = 0;
delay_10ms(50);

// INT0 / INT1 konfigurieren
//	EICRA = (0<<ISC11 | 0<<ISC10 | 0<<ISC01 | 0<<ISC00);  // low_level Interrupt
	
// nicht bentigte Module abschalten um Strom zu sparen
   PRR = (0<<PRTWI | 1<<PRTIM2 | 1<<PRTIM0 | 1<<PRTIM1 | 1<<PRSPI | 0<<PRUSART0 | 1<<PRADC);
   ACSR = (1<<ACD);                       // Analog Comparator abschalten

asm volatile("sei");								// fr TWI und RX erforderlich !!

// TWI-/RS232-/PCF8583-Init
rs232_init();
puts_p(PSTR(LOGON_TXT));						// Startmeldung auf COM: ausgeben 
put_c(13);

TWI_Master_Initialise();
PCF8583_init();									// hier wird auch die Zeit gelesen

// --- TWI-Busdevices auf Anwesenheit testen

// es wurde auf den PCF8583 zugegriffen
// wenn ein Fehler vorliegt, dann mit blinkender LED beenden
if (!(TWI_Get_State_Info() == TWI_NO_STATE))
{
	DDRD = (1<<LED_KONFIG);
	while(1)
	{
		delay_10ms(25);
		PIND = (1<<LED_KONFIG);
	}
}
/*
puts_p(PSTR("PCF8583 ok"));
put_c(13);
*/

// testen, ob das EEprom ansprechbar ist
buffer[0] = TWI_EEPROM_ADR;
buffer[1] = 0;
buffer[2] = 0;

TWI_MA_Start_Transceiver_With_Data(buffer, 3);
if (!(TWI_Get_State_Info() == TWI_NO_STATE))
{
	DDRD = (1<<LED_LOG);
	while(1)
	{
		delay_10ms(25);
		PIND = (1<<LED_LOG);
	}
}
/*
puts_p(PSTR("AT24C512 ok"));
put_c(13);
*/
// --- im Eeprom des m88 gespeicherte Daten auslesen: Anzahl Sensoren bestimmen
OW_read_eeprom();		
puts_p(PSTR("Sensoren angemeldet: "));
put_s(utoa(ow_sensor_cnt, cv, 10));
put_c(13);

// wenn nicht alle angemeldeten Sensoren gefunden werden, dann mit beiden LED's
// blinken im Gegentakt fr 10 Sekunden
if (ow_sensor_cnt && (!(OW_test_sensors())))
	{
	i = 20;
	DDRD = (1<<LED_LOG | 1<<PIN_INT0);
	PIND = (1<<LED_LOG);

	puts_p(PSTR("Sensor-Fehler"));
	put_c(13);

	while(i--)
		{
		delay_10ms(25);
		PIND = (1<<LED_LOG | 1<<LED_KONFIG);
		}
	}

// --- nun gehts an die Arbeit -----------------------------------------------
	while(1)
	{
		PCF8583_konfig();							// zuerst ins Konfig-Menue
		start_logging();							// Loggen bis zum (manuellen) Abbruch
	}
}

// ----------------------------------------------------------------------------
// wartet fr die Dauer eines Messintervalls und stt dann die Messungen an (INT0)
// alternativ kann mit einem Tastendruck INT1 auf Masse gezogen werden
// dann wird in die Konfig-Routine gesprungen

void start_logging(void)
{
   DDRD   =  (1<<LED_LOG);               	// LED von + ber LEG_LOG nach gnd geschaltet
	PORTD  = (uint8_t) ~(1<<LED_LOG);						// Led-Pin auf gnd 

   // die Interrupt-Flags und das Messintervall des PCF8583 zurcksetzen
	PCF8583_timer_enable();
	
	// Korrekturwert fr die Uhr in lokale Variablen im Modul PCF8583 schreiben
	PCF8583_make_korrektur_values();
	
	delay_10ms(100);								// die LED_LOG fr 1 Sekunden leuchten lassen
//	messung();										// die erste Messung ausfhren mit aktiver LED_LOG
	
// nun alle LEDs abschalten
	DDRD  = 0;										// gesamten Port auf Eingang schalten
	PORTD = 0xFF;									// alle Pullups einschalten
	
	while(1)
	{
		while ( TWCR & (1<<TWIE) );			// abwarten, bis letzte TWI-bertragung abgeschlossen ist
		while(!(UCSR0A & (1<<UDRE0)));		// abwarten, bis letzte RS232-bertragung abgeschlossen ist
		delay_10ms(2);								// kurze Verzgerung vor dem sleep einlegen
	
   // Interrupts enablen
	//	EICRA = (0<<ISC11 | 0<<ISC10 | 0<<ISC01 | 0<<ISC00);  // low_level Interrupt
		EIFR = 0;									// falls Flags gesetzt sind
		EIMSK = (1<<INT0 | 1<<INT1);    		// INT0/INT1 aktivieren
		SMCR =  (0<<SM0  | 1<<SM1 | 0<<SM0 | 1<<SE); // Power-Down Modus, Sleep enable

   // den Sleep-Modus aktivieren und auf einen low-level-Interrupt warten
		asm volatile("sleep");

	// ---- an dieser Stelle wird auf einen low-level-Interrupt an INT0/INT1 gewartet

		EIMSK = 0;                      		// INT0/INT1 abschalten

		if (int_source)                  	// Interrupt1 hat ausgelst - Taste bettigt
		{
			int_source = 0;
//			messung();								// eine letzte Messung ausfhren

			PORTD  = (uint8_t) ~(1<<LED_LOG);				// LED_LOG einschalten

			PCF8583_timer_disable();
														// falls der Taster zu lange bettigt wird,
														// startet das Programm gleich wieder durch, daher
			delay_10ms(25);						// 250ms warten
			while (!(PIND & (1<<PIN_INT1))); // warten, bis der Pin losgelassen wird
			
			return;                       	// beenden, zurck zum Men
		}

		else                             	// Interrupt0 hat ausgelst - Zeitintervall abgelaufen
		{
			messung();								// eine Messreihe ausfhren und die Messwerte speichern
		}
	}
}

// ----------------------------------------------------------------------------
// den PCF8583 im Bildschirmdiaglog mittels Terminalprogramm mit Daten fttern:
// Systemzeit/datum anzeigen/ndern, Messintervall eingeben
void PCF8583_konfig(void)
{
   uint8_t temp, input, exit = 0;
	int8_t itemp;
	
   DDRD    =  (1<<LED_KONFIG);				// Pin fr LED's als Ausgang setzen
	PORTD   =  (1<<PIN_INT1);					// Pullup fr Taster/INT1, alle anderen Pins auf low
														// LED_KONFIG zeigt an, dass konfiguriert wird
	PCF8583_timer_disable();
   while(!exit)
   {
		PCF8583_read_time();						// aktuelle Zeit einlesen
		PCF8583_read_date();
		
		put_c(13);
      puts_p(PSTR("Aktuelle Zeit: "));
      put_s(itoa(time[TAG], cv, 10));
      put_c('.');
      put_s(itoa(time[MONAT], cv, 10));
      put_c('.');
      put_s(itoa(time[JAHR], cv, 10));
      put_c(' ');
      put_s(itoa(time[STUNDE], cv, 10));
      put_c(':');
      put_s(itoa(time[MINUTE], cv, 10));
      put_c(':');
      put_s(itoa(time[SEKUNDE], cv, 10));
      put_c(13);
		
	// wenn Korrekturwerte fr die Uhr eingegeben sind, dann diese ausdrucken
		itemp = PCF8583_read_byte(PCF_CLOCK_KORREKTUR);
		if(!(itemp == 0))
			{
			puts_p(PSTR("Zeitkorrektur: "));
			put_s(itoa(itemp, cv,10));
			puts_p(PSTR(" * 1/100 sec/h"));
			put_c(13);
			}
			
	// das Messintervall ausgeben
      puts_p(PSTR("Messintervall: "));
		if (mess_intervall) put_s(itoa(mess_intervall, cv, 10));
		else puts_p(PSTR("Kein Zeitintervall definiert "));
      
		if (mess_einheit == 'M') put_c('m');
      else if (mess_einheit == 'S') put_c('s');
		else puts_p(PSTR("- keine Zeiteinheit definiert !"));
      
		put_c(13);
      put_c(13);
      puts_p(PSTR("Datum/Uhrzeit einstellen (1)"));
      put_c(13);
      puts_p(PSTR("Messintervall einstellen (2)"));
		put_c(13);
      puts_p(PSTR("Neue Messreihe starten   (3)"));
      put_c(13);
		puts_p(PSTR("Messwerte exportieren    (4)"));
      put_c(13);
      puts_p(PSTR("Messreihe fortsetzen     (5)"));
      put_c(13);
      puts_p(PSTR("DS18B20 konfigurieren    (0)"));
      put_c(13);
		put_c(13);
      puts_p(PSTR("Bitte waehlen: "));

    // warten auf die Eingabe eines einzelnen Zeichens
		input = ' ';		// als Vorgabe ein ungltiges/unsichtbares Zeichen whlen
		get_single_char(&input);

    // Eingabe auswerten
	 	switch (input)
		{
			case '1':
				get_time();							// die Zeit abfragen
				// die clock zhlt die Jahre nur von 0..3
				// die Differenz wird im RAM des PCF8583 abgelegt.
				PCF8583_write_byte(PCF_YEAR_OFFSET, (time[JAHR] - (time[JAHR] % 4)));
				time[SEK_100] = 0;				// 1/100 tel Sekunde auf 0
				PCF8583_write_time();			// die Werte in die clock bertragen
				PCF8583_write_date();
				break;

			case '2':
			// Zeiteinheit (Sekunden/Minuten) abfragen
				put_c(13);
				while(1)
				{
					puts_p(PSTR("Einheit Messintervall (Sec/Min) ["));
					if (mess_einheit) put_c(mess_einheit); else put_c('?');
					put_c(']');

					get_single_char(& mess_einheit);
					mess_einheit &= (~(0x20));
		
					if ((mess_einheit == 'S') || (mess_einheit == 'M')) break;
					else 
					{
						put_c(' ');
						put_c(mess_einheit);
						puts_p(PSTR(" invalid"));
					}
				}
				put_c(13);

			// das Messintervall anzeigen und abfragen
				puts_p(PSTR("Intervall in "));
				if (mess_einheit == 'S') puts_p(PSTR("Sekunden ["));
				else if (mess_einheit == 'M') puts_p(PSTR("Minuten ["));
				put_s(itoa(mess_intervall, cv, 10));
				puts_p(PSTR("] :"));
				get_value(&mess_intervall, 1, 99);
				put_c(13);
			
			// die Werte im RAM des PCF ablegen
				PCF8583_write_byte(PCF_ALARM_UNIT,      mess_einheit);
				PCF8583_write_byte(PCF_ALARM_INTERVALL, mess_intervall);
				break;

			case '3':
			// hier wird nur der Adresscounter im RAM des PCF8583 'genullt'.
			// Der Inhalt des Eeproms bleibt unangetastet.
            TWI_eeprom_reset();
				exit++;
				break;

			case '4':
			// Export von Daten
				if (twi_eeprom_adr == 0)		// dann sind 'logisch' keine Daten vorhanden
				{
					puts_p(PSTR("keine Daten"));
					put_c(13);
					break;
				}
			// entscheiden, welche Datenreihen exportiert werden sollen
				put_c(13);
				puts_p(PSTR("Exportmaske : [255] "));
				input = 255;
				get_value(&input, 1, 255);
				
				put_c(13);
				puts_p(PSTR("Start mit RETURN"));
				put_c(13);									

				ENABLE_RX();						// warten auf ein RETURN
				rx_clear_buffer();				// hier wird u.a. rx_flag gelscht
				while (!rx_flag);             // wartet bis ein RETURN eingegeben wird 
				DISABLE_RX();
			
				print_messreihe(input);			// hier erfolgt der Export der Daten
				
				ENABLE_RX();						// warten auf ein RETURN
				rx_clear_buffer();				// hier wird u.a. rx_flag gelscht
				while (!rx_flag);             // wartet bis ein RETURN eingegeben wird 
				DISABLE_RX();
				
				break;

			case '5':
				exit++;								// exit > 0 beendet die Eingabeschleife
				break;

			case '0':
				OW_config();						// Sprung ins Menue fr die Temperatursensoren
				break;

// --- Start undokumentierte Funktionen ---------------------------------------			

#ifdef DEBUG
			case 'e':								// 'e' wie Eeprom - gibt den Inhalt des Eeproms bis zum
				test_eeprom(FALSE);				// ersten 0xFF im ersten Byte eines Datensatzes aus
				break;								// ndert die twi_eeprom_adr nicht

			case 'r':								// 'r' wie recover - wie 'e', zustzlich wird die Adresse des letzten Datensatzes
														// als nchste Datensatzadr auf den PCF8583 geschrieben.
														// Gedacht fr den Fall, das der PCF8583 seine Daten vergessen hat.
														// Dann ist kein Export mglich, lediglich ein Auszug aus dem Eeprom
				test_eeprom(TRUE);
				break;

			case 'p':
				test_pcf();							// 'p' wie PCF8583 - gibt die Register und das benutzte RAM
				break;								// des PCF8583 aus
#endif
			case 'D':								// 'D' wie Delete - lscht das TWI-Eeprom durch berschreiben mit 0xFF
				puts_p(PSTR("TWI-Eeprom loeschen mit 'X'")); 
				input = ' ';
				get_single_char(& input);
				if (input == 'X' )				// 'X' wie eXecute
					{
					puts_p(PSTR("loesche ..."));
					TWI_eeprom_clear();
					puts_p(PSTR(" fertig !"));
					}
				put_c(13);
				break;
				
			case 'T':								// 'T' wie TimeKorrektur
														// Um die Gangabweichung der Uhr auszugleichen, kann
														// hier die Abweichung in Sekunden pro Tag angeben werden
														// Mit dem Korrekturwert wird stndlich mit der Zeit angepasst
				put_c(13);
				puts_p(PSTR("Zeitkorrektur 1/100 sec/h : "));
				put_s(itoa(itemp, cv, 10)); 	// Ausgabe des int8-Wertes mit Vorzeichen
				put_c(13);
				
				temp = (uint8_t) abs(itemp);	// um spter den absoluten Wert abzufragen
				
				puts_p(PSTR("Vorzeichen +/- : ["));// Abfrage des Vorzeichens

				if (itemp < 0) input = '-'; else input = '+';
				put_c(input);
				put_c(']');
				put_c(' ');
				
				while (1)					// solange, bis + oder - eingegeben wurde
					{
					get_single_char(&input);
					if ((input == '-') || (input == '+'))
						{
						break;
						put_c(13);
						}
					
					else
						{
						puts_p(PSTR("Eingabe +/-"));
						put_c(13);	
						}
					}
				puts_p(PSTR("Wert absolut : ["));	// Abfrage des absolute-Wertes
				put_s(utoa(temp, cv, 10));
				put_c(']');
				put_c(' ');
				
				get_value(&temp, 0, 127);			// erwartet uint8_t als Pointer
															// uint8_t in int8_t umwandeln
				if (input == '-')	itemp = (int8_t) (0 - temp);
				else itemp = (int8_t) temp;
				
				// den Werte im RAM des PCF ablegen
				PCF8583_write_byte(PCF_CLOCK_KORREKTUR, itemp);

				// den Wert wieder auslesen und im Modul PCF8583.c lokale Variablen schreiben
				PCF8583_make_korrektur_values();
				break;

// --- End undokumentierte Funktionen ------------------------------------------
				
			default:
				put_c(13);
				puts_p(PSTR("ungueltige Eingabe"));
		}
	// Solange keine Messintervall und keine mess_einheit definiert sind,
	// darf das Konfig-Menue nicht verlassen werden
	if ((mess_intervall == 0) || (mess_einheit == 0))
		{
		exit = 0;									// fr while(!exit)
		}
   }
	
	// sicherstellen, dass nicht mehr Sensoren angemeldet sind als Messwerte
	// in einem Datensatz Platz finden
	// berzhliges abschneiden
	if (ow_sensor_cnt > TWI_REC_FIELD_CNT) ow_sensor_cnt = TWI_REC_FIELD_CNT;
}

// ----------------------------------------------------------------------------
// Fragt Datum und Uhrzeit ab.
// Die aktuellen Werte werden in [.] angezeigt und knnen mit RETURN besttigt werden
// Die Eingabereihenfolge ist
// 1. Jahr -> damit ist geklrt, ob es sich um ein Schaltjahr handelt
// 2. Monat -> mit der Information zum Schaltjahr ist die Anzahl der Tage definiert
// 3. Tag 
// Es erfolgt eine Prfung auf den jeweils gltigen Wertebereiche

void get_time(void)
{
	uint8_t t;
	
   put_c(13);
   put_c(13);
	
   puts_p(PSTR("Jahr   : ["));
	put_s(utoa(time[JAHR],cv,10));
	put_s("] ");
   get_value(&time[JAHR], 0, 59);		// Das Jahr ist nur bis 2063 speicherbar ! 
   put_c(13);

// das Jahr kann auf dem PCF8583 nur mit [0..3] dargestellt werden
//	die Differenz zum Jahr 2000 wird im RAM des PCF8583 gespeichert
	   
	puts_p(PSTR("Monat  : ["));
	put_s(utoa(time[MONAT],cv,10));
	put_s("] ");
   get_value(&time[MONAT], 1, 12);
   put_c(13);

// die Anzahl der Tage des Monats ermitteln
// Schaltjahre werden dabei bercksichtigt.
   t = days_of_month(time[MONAT]);
	
	puts_p(PSTR("Tag    : ["));
	put_s(utoa(time[TAG],cv,10));
	put_s("] ");
   get_value(&time[TAG], 1, t);
   put_c(13);

   puts_p(PSTR("Stunde : ["));
	put_s(utoa(time[STUNDE],cv,10));
	put_s("] ");
   get_value(&time[STUNDE],0, 23);
   put_c(13);

   puts_p(PSTR("Minute : ["));
	put_s(utoa(time[MINUTE],cv,10));
	put_s("] ");
   get_value(&time[MINUTE], 0, 59);
   put_c(13);
	
   puts_p(PSTR("Sekunde: ["));
	put_s(utoa(time[SEKUNDE],cv,10));
	put_s("] ");
   get_value(&time[SEKUNDE], 0, 59);
   put_c(13);
}

// ----------------------------------------------------------------------------
// im Hintergrund ist eine Interrupt-gesteuerte Eingabe aktiv.
// Die eingehenden Daten werden in einem Puffer abgelegt.
// Sobald ein RETURN empfangen wird, wird das Flag rx_flag gesetzt.

// die Variable, die beschrieben werden soll, wird per Pointer bergeben
// - bei Eingabe von ENTER bleibt der alte Wert unverndert
// - Eingaben mit ungltigen Zeichenfolgen werden verworfen

void get_value(uint8_t * data, uint8_t min, uint8_t max)
{
   uint8_t s, r, j;
	uint8_t exit = 0;
	
	ENABLE_RX();
	
	while(!exit)
		{
		rx_clear_buffer();						// hier wird u.a. rx_flag gelscht
		while (!rx_flag);                  	// wartet darauf, dass eine Zeichenfolge 
														// mit Return abgeschlossen wird
		r = rx_count_bytes();               // nachsehen, wieviele Bytes eingegangen sind

		if (!r) 										// wenn nur ENTER eingegeben wurde (r == 0),
			{ 											// dann wird data nach s kopiert
			s = *data;								// fr die anschlieende Bereichsprfung von s ..
			exit++;									// damit die while(exit) Schleife beendet wird
			}

		else
			{
			s = 0;

			while (r)								// r wird erst am Ende der Schleife decrementiert
				{										// damit der Test if(!r) funktioniert !!
				j = rx_get_byte();            // das nchste Zeichen holen
				if (!(isdigit(j)))
					{									// Fehlermeldung falls keine Ziffer 
					puts_p(PSTR(" error char : "));
					put_c(j);
					put_c(13);
					break;
					}
				j -= '0';                     // den Zahlenwert aus der Ziffer destillieren
				s *= 10;                      // und eine Dezimalzahl daraus bilden
				s += j;
				r--;
				}
			}
		
		if (!r) 										// wenn keine Fehler aufgetreten, wird geprft,
		{												// ob der Wert im zulssigen Bereich liegt
			if((s >= min) && (s <= max)) exit++;
			else
			{
				exit = 0;							// wenn mit RETURN ein ungltiger Wert besttigt wurde
				puts_p(PSTR(" out of range : "));
				put_s(utoa(s ,cv ,10));			// den Fehlerwert ausgeben
				put_c(13);
			}
		}
	}
	put_s(utoa(s,cv,10));						// Echo ausgeben
   *data = s;										// hier wird der neue Wert zugewiesen
	DISABLE_RX();
 }

// ----------------------------------------------------------------------------
// Wartet darauf, dass genau ein Zeichen eingegeben und mit RETURN abgeschlossen wird.
// Liefert das eingegebene Zeichen zurck
// Ein lowlevel INT1 emuliert die Terminaleingabe '5' 
void get_single_char(uint8_t * vorgabe)
{
	uint8_t i;
	ENABLE_RX();

	EIMSK = (0<<INT0 | 1<<INT1); 				// INT1 aktivieren fr den Taster

	while(1)
   {
      rx_clear_buffer();
      while (!(rx_flag) && !(int_source));// auf ein RETURN bzw. den Taster warten:
														// ein RETURN auf der Tastatur setzt rx_flag auf 1,
														// der Taster an INT1 setzt int_source auf 1
		if (int_source)							// wenn der Taster gedrckt wurde ...
		{
			EIMSK = (0<<INT0 | 0<<INT1);		// damit eine prellende Taste keinen weiteren INT auslst
			int_source = 0;						// das Flag zurcksetzen
			delay_10ms(20);						// wegen der prellenden Taste warten
			while (!(PIND & (1<<PIN_INT1)));	// weiter warten, solange Taste gedrckt
			*vorgabe = '5';						// Tastatureingabe fr 'Messung fortsetzen'
			break;
		}
		
		else											// wenn RETURN gedrckt wurde
		{
			i = rx_count_bytes();
		
			if (i == 0)								// Vorgabe wurde mit Enter besttigt
			{
				break;
			}
			if (i == 1)								// genau ein Zeichen eingegeben
			{
				*vorgabe = rx_get_byte();
				break;
			}

			else
			{
				puts_p(PSTR("genau 1 Zeichen eingeben"));
				put_c(13);
			}
		}
	}
   DISABLE_RX();
	EIMSK = (0<<INT0 | 0<<INT1);				// den INT1 deaktivieren
	put_c(*vorgabe);								// ein Echo der Eingabe ausgeben
	put_c(13);
}

// ----------------------------------------------------------------------------
// INT0-Pin ist auf low gezogen worden
// der Timer PCF8583 hat den Interrupt ausgelst

EMPTY_INTERRUPT(INT0_vect)

// ----------------------------------------------------------------------------
// INT1-Pin ist auf low gezogen worden
// die Taste zur Konfiguration wurde bettigt
// der Interrupt wird anschlieend sofort disabled, prellende Kontakte bleiben folgenlos

ISR(INT1_vect)
{
   int_source = 1;                        // damit erkannt wird, ob INT0 oder INT1 
}														// das Sleep beendet hat

// ----------------------------------------------------------------------------
// Eine Verzgerung um t * 10ms
void delay_10ms(uint8_t t)
{
while(t--) _delay_ms(10);
}

// --- [ eof ] ----------------------------------------------------------------
